@@ -1,7 +1,7 @@
 /*
     meta.h -- header for meta.c
-    Copyright (C) 2000-2005 Guus Sliepen <guus@tinc-vpn.org>,
-                  2000-2005 Ivo Timmermans <ivo@tinc-vpn.org>
+    Copyright (C) 2000-2006 Guus Sliepen <guus@tinc-vpn.org>,
+                  2000-2005 Ivo Timmermans
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -17,7 +17,7 @@
     along with this program; if not, write to the Free Software
     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 
-    $Id: meta.h 1439 2005-05-04 18:09:30Z guus $
+    $Id: meta.h 1452 2006-04-26 13:52:58Z guus $
 */
 
 #ifndef __TINC_META_H__
@@ -27,6 +27,7 @@
 
 extern bool send_meta(struct connection_t *, const char *, int);
 extern void broadcast_meta(struct connection_t *, const char *, int);
+extern bool flush_meta(struct connection_t *);
 extern bool receive_meta(struct connection_t *);
 
 #endif							/* __TINC_META_H__ */
