@@ -1,7 +1,7 @@
 /*
     net_socket.c -- Handle various kinds of sockets.
-    Copyright (C) 1998-2005 Ivo Timmermans <ivo@tinc-vpn.org>,
-                  2000-2005 Guus Sliepen <guus@tinc-vpn.org>
+    Copyright (C) 1998-2005 Ivo Timmermans,
+                  2000-2006 Guus Sliepen <guus@tinc-vpn.org>
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -17,7 +17,7 @@
     along with this program; if not, write to the Free Software
     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 
-    $Id: net_socket.c 1439 2005-05-04 18:09:30Z guus $
+    $Id: net_socket.c 1460 2006-08-08 13:50:58Z guus $
 */
 
 #include "system.h"
@@ -38,16 +38,43 @@
 #define EINPROGRESS WSAEINPROGRESS
 #endif
 
+/* Needed on Mac OS/X */
+#ifndef SOL_TCP
+#define SOL_TCP IPPROTO_TCP
+#endif
+
 int addressfamily = AF_UNSPEC;
 int maxtimeout = 900;
 int seconds_till_retry = 5;
-bool blockingtcp = false;
 
 listen_socket_t listen_socket[MAXSOCKETS];
 int listen_sockets;
 
 /* Setup sockets */
 
+static void configure_tcp(connection_t *c)
+{
+	int option;
+
+#ifdef O_NONBLOCK
+	int flags = fcntl(c->socket, F_GETFL);
+
+	if(fcntl(c->socket, F_SETFL, flags | O_NONBLOCK) < 0) {
+		logger(LOG_ERR, _("fcntl for %s: %s"), c->hostname, strerror(errno));
+	}
+#endif
+
+#if defined(SOL_TCP) && defined(TCP_NODELAY)
+	option = 1;
+	setsockopt(c->socket, SOL_TCP, TCP_NODELAY, &option, sizeof(option));
+#endif
+
+#if defined(SOL_IP) && defined(IP_TOS) && defined(IPTOS_LOWDELAY)
+	option = IPTOS_LOWDELAY;
+	setsockopt(c->socket, SOL_IP, IP_TOS, &option, sizeof(option));
+#endif
+}
+
 int setup_listen_socket(const sockaddr_t *sa)
 {
 	int nfd;
@@ -64,33 +91,11 @@ int setup_listen_socket(const sockaddr_t *sa)
 		return -1;
 	}
 
-#ifdef O_NONBLOCK
-	{
-		int flags = fcntl(nfd, F_GETFL);
-
-		if(fcntl(nfd, F_SETFL, flags | O_NONBLOCK) < 0) {
-			closesocket(nfd);
-			logger(LOG_ERR, _("System call `%s' failed: %s"), "fcntl",
-				   strerror(errno));
-			return -1;
-		}
-	}
-#endif
-
 	/* Optimize TCP settings */
 
 	option = 1;
 	setsockopt(nfd, SOL_SOCKET, SO_REUSEADDR, &option, sizeof(option));
 
-#if defined(SOL_TCP) && defined(TCP_NODELAY)
-	setsockopt(nfd, SOL_TCP, TCP_NODELAY, &option, sizeof(option));
-#endif
-
-#if defined(SOL_IP) && defined(IP_TOS) && defined(IPTOS_LOWDELAY)
-	option = IPTOS_LOWDELAY;
-	setsockopt(nfd, SOL_IP, IP_TOS, &option, sizeof(option));
-#endif
-
 	if(get_config_string
 	   (lookup_config(config_tree, "BindToInterface"), &iface)) {
 #if defined(SOL_SOCKET) && defined(SO_BINDTODEVICE)
@@ -241,15 +246,7 @@ void finish_connecting(connection_t *c)
 
 	ifdebug(CONNECTIONS) logger(LOG_INFO, _("Connected to %s (%s)"), c->name, c->hostname);
 
-#ifdef O_NONBLOCK
-	if(blockingtcp) {
-		int flags = fcntl(c->socket, F_GETFL);
-
-		if(fcntl(c->socket, F_SETFL, flags & ~O_NONBLOCK) < 0) {
-			logger(LOG_ERR, _("fcntl for %s: %s"), c->hostname, strerror(errno));
-		}
-	}
-#endif
+	configure_tcp(c);
 
 	c->last_ping_time = now;
 
@@ -259,7 +256,7 @@ void finish_connecting(connection_t *c)
 void do_outgoing_connection(connection_t *c)
 {
 	char *address, *port;
-	int option, result, flags;
+	int result;
 
 	cp();
 
@@ -314,25 +311,7 @@ begin:
 
 	/* Optimize TCP settings */
 
-#if defined(SOL_TCP) && defined(TCP_NODELAY)
-	option = 1;
-	setsockopt(c->socket, SOL_TCP, TCP_NODELAY, &option, sizeof(option));
-#endif
-
-#if defined(SOL_IP) && defined(IP_TOS)
-	option = IPTOS_LOWDELAY;
-	setsockopt(c->socket, SOL_IP, IP_TOS, &option, sizeof(option));
-#endif
-
-	/* Non-blocking */
-
-#ifdef O_NONBLOCK
-	flags = fcntl(c->socket, F_GETFL);
-
-	if(fcntl(c->socket, F_SETFL, flags | O_NONBLOCK) < 0) {
-		logger(LOG_ERR, _("fcntl for %s: %s"), c->hostname, strerror(errno));
-	}
-#endif
+	configure_tcp(c);
 
 	/* Connect */
 
@@ -409,7 +388,8 @@ bool handle_new_meta_connection(int sock)
 {
 	connection_t *c;
 	sockaddr_t sa;
-	int fd, len = sizeof(sa);
+	int fd;
+	socklen_t len = sizeof(sa);
 
 	cp();
 
@@ -437,15 +417,7 @@ bool handle_new_meta_connection(int sock)
 
 	ifdebug(CONNECTIONS) logger(LOG_NOTICE, _("Connection from %s"), c->hostname);
 
-#ifdef O_NONBLOCK
-	if(blockingtcp) {
-		int flags = fcntl(c->socket, F_GETFL);
-
-		if(fcntl(c->socket, F_SETFL, flags & ~O_NONBLOCK) < 0) {
-			logger(LOG_ERR, _("fcntl for %s: %s"), c->hostname, strerror(errno));
-		}
-	}
-#endif
+	configure_tcp(c);
 
 	connection_add(c);
 
