@@ -1,7 +1,7 @@
 /*
     tincd.c -- the main file for tincd
-    Copyright (C) 1998-2005 Ivo Timmermans <ivo@tinc-vpn.org>
-                  2000-2005 Guus Sliepen <guus@tinc-vpn.org>
+    Copyright (C) 1998-2005 Ivo Timmermans
+                  2000-2006 Guus Sliepen <guus@tinc-vpn.org>
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -17,7 +17,7 @@
     along with this program; if not, write to the Free Software
     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 
-    $Id: tincd.c 1439 2005-05-04 18:09:30Z guus $
+    $Id: tincd.c 1466 2006-11-11 20:10:46Z guus $
 */
 
 #include "system.h"
@@ -35,8 +35,14 @@
 #include <openssl/rsa.h>
 #include <openssl/pem.h>
 #include <openssl/evp.h>
+#include <openssl/engine.h>
 
+#ifdef HAVE_LZO_LZO1X_H
+#include <lzo/lzo1x.h>
+#endif
+#ifdef HAVE_LZO1X_H
 #include <lzo1x.h>
+#endif
 
 #include <getopt.h>
 #include "pidfile.h"
@@ -407,7 +413,7 @@ int main(int argc, char **argv)
 	if(show_version) {
 		printf(_("%s version %s (built %s %s, protocol %d)\n"), PACKAGE,
 			   VERSION, __DATE__, __TIME__, PROT_CURRENT);
-		printf(_("Copyright (C) 1998-2005 Ivo Timmermans, Guus Sliepen and others.\n"
+		printf(_("Copyright (C) 1998-2006 Ivo Timmermans, Guus Sliepen and others.\n"
 				"See the AUTHORS file for a complete list.\n\n"
 				"tinc comes with ABSOLUTELY NO WARRANTY.  This is free software,\n"
 				"and you are welcome to redistribute it under certain conditions;\n"
@@ -448,6 +454,9 @@ int main(int argc, char **argv)
 
 	RAND_load_file("/dev/urandom", 1024);
 
+	ENGINE_load_builtin_engines();
+	ENGINE_register_all_complete();
+
 	OpenSSL_add_all_algorithms();
 
 	if(generate_keys) {
@@ -505,6 +514,8 @@ end:
 #ifndef HAVE_MINGW
 	remove_pid(pidfilename);
 #endif
+
+	EVP_cleanup();
 	
 	return status;
 }
